#! /bin/sh

usage()
{
	cat <<EOF
Usage:
	$0 --initrd <debian-initrd> <destination-hacked-initrd>
	$0 --root /path/to/vsa/root/mountpoint

Exemple:
	$0 --initrd /boot/initrd.img-2.6.32-5-xen-amd64 \\
		/boot/initrd.img-2.6.32-5-xen-amd64-vsa

	mount /dev/sda5 /mnt &&
	$0 --root /mnt

EOF
	exit 0
}

die()
{
	cat <<EOF
ERROR: $@
EOF
	exit 1
}

hack_mobo()
{
	file="$1"/etc/init.d/lhn_functions
	# already hacked ?
	grep "VSA-XEN-HACK" "$file" > /dev/null && return
	sed -i '/moboUUID=/a\
	# VSA-XEN-HACK\
        [ "$moboUUID" ] || {\
		mac=`/sbin/ifconfig eth0 | \
			sed -n "/HW/{s/ *$//;s/.* //;s/://g;p;}"`\
		[ "$mac" ] || mac="deadb33fc0de"\
		# need to return the same value over reboot\
		# first part was generated with uuidgen\
		moboUUID="160d007e-0f9e-11e1-b59f-$mac"\
	}' "$file"
	echo "motherboard uuid hacked. ($file)"
}
hack_pci_dir()
{
	dir="$1"/hack/bus/pci
	[ -f "$dir"/devices ] && return
	mkdir -p "$dir"
	cat > "$dir"/devices <<"EOF" && chmod 444 "$dir"/devices
0000	58530001	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0
EOF
	echo "pci/devices hacked. ($dir)"
}
hack_dev_scsi()
{
	dir="$1"/etc/udev
	[ -f "$dir"/rules.d/53-devfs-hack.rules ] && return
	cat > "$dir"/rules.d/53-devfs-hack.rules <<"EOF"
KERNEL=="xvd*", PROGRAM="/etc/udev/scripts/xvd-hack.sh %k", SYMLINK+="scsi/%c", MODE="0600" 
EOF
	script="$dir"/scripts/xvd-hack.sh
	cat <<"EOF" > "$script" && chmod +x "$script"
#! /bin/sh

DEV="$1"
case "$DEV" in
        xvd[abcdefg]|xvd[abcdefg][0-9]) : ;;
        *) exit 1 ;;
esac

case "$DEV" in
        xvd[ab]*) host=0 ;;
        *) host=1 ;;
esac
case "$DEV" in
        xvd[ac]*) target=0 ;;
        xvdd*) target=1 ;;
        xvd[be]*) target=2 ;;
        xvdf*) target=3 ;;
        xvdg*) target=4 ;;
esac
ext="${DEV#xvd?}"
if [ "$ext" ] ; then
        ext=part"$ext"
else
        ext="disc"
fi
echo host$host/bus0/target$target/lun0/$ext
EOF
	echo "dev/scsi hacked. ($dir)"
}
hack_modules()
{
	ver=`uname -r`
	dir="$1"/lib/modules/"$ver"
	[ -d "$dir" ] && return
	rsync -a /lib/modules/"$ver"/ "$dir"/
	echo "modules included ($dir)"
}
hack_brand()
{
	dir="$1"/etc/lefthand/brand/VMNSM
	file="$dir"/brand.init
	grep "VSA-XEN-HACK" "$file" > /dev/null && return
	sed -i '/function base_init/,/^}/{
		/case/a\
	250) # VSA-XEN-HACK\
	    printLog "Configuring VSA for Xen"\
	    # Disks are the same as vmware\
            ln -sf ${CONFIG_DIR}/platform.xml.vmw ${CONFIG_DIR}/platform.xml\
	    # We do not use boot from VSA, so we do not care for those\
	    ln -sf ${CONFIG_DIR}/linuxrc.xen ${CONFIG_DIR}/linuxrc\
	    ln -sf ${CONFIG_DIR}/syslinux.cfg.xen ${CONFIG_DIR}/syslinux.cfg\
	;;
		}' "$file"
	file="$dir"/rc.drivers
	sed -i '/elif/,/else/{
		/naction/s/^/#/
		}
 /^else/i\
	# VSA-XEN-HACK\
        # Note: all might not even be needed\
        # we already have disks if we arrive here,\
        # and most likely network is also up\
        naction "Loading block Driver:"    modprobe xen_blkfront\
        naction "Loading Network Driver:"  modprobe xen_netfront\
        naction "Loading Network Driver:"  modprobe ext3\
        naction "Loading Ramdisk Driver:"  modprobe rd rd_size=32768\
        naction "Loading raid Driver:"     modprobe md\
        naction "Loading raid Driver:"     modprobe raid0\
        naction "Loading raid Driver:"     modprobe raid1
		' "$file"
	echo "brand hacked ($dir)"
}
hack_inittab()
{
	file="$root"/etc/inittab
	grep "VSA-XEN-HACK" "$file" > /dev/null && return
	sed -i 's/^\([1-5]\|s0\):/#&/;
		/^6:/{
			s/^/#/
			a\
# VSA-XEN-HACK\
7:2345:respawn:/sbin/agetty -l /bin/su hvc0 38400 linux
		}' "$file"
	echo "inittab hacked. ($file)"
}

hack_root()
{
	root="$1"
	[ -d "$root"/etc/lefthand ] ||
		die "etc/lefthand not found. This is not a valid VSA root path."
	hack_mobo "$root"
	hack_pci_dir "$root"
	hack_dev_scsi "$root"
	hack_brand "$root"
	hack_modules "$root"
	hack_inittab "$root"
}

hack_init()
{
	initsrc="$1"
	initdst="$2"
	[ -f "$initsrc" ] || die "$initsrc: file not found"
	[ -f "$initdst" ] && die "$initdst: file already exists"
	case "$initsrc" in
		/*) : ;;
		*) die "$initsrc: need to be an absolute path"
	esac
	case "$initdst" in
		/*) : ;;
		*) die "$initdst: need to be an absolute path"
	esac
	dir=`mktemp -d`
	cd "$dir" || die "can't cd to $dir"
	gzip -d < "$initsrc" | cpio -i
	file=scripts/init-bottom/pcihack
	cat <<"EOF" > "$file" && chmod +x "$file"
#! /bin/sh
[ -d "/root/hack/bus/pci" ] &&
mount -o bind /root/hack/bus/pci /proc/bus/pci
EOF
	echo 
	[ -f scripts/init-bottom/ORDER ] &&
	sed -i '1i/scripts/init-bottom/pcihack' scripts/init-bottom/ORDER
	find . -print0 | cpio -0 -H newc -o | gzip -c > "$initdst"
	cd /
	rm -rf "$dir"
}


umask 022
case "$1" in
	--root) 
		[ "$#" -eq 2 ] || usage
		hack_root "$2" ;;
	--initrd)
		[ "$#" -eq 3 ] || usage
		hack_init "$2" "$3" ;;
	*) usage
esac
